@@ -11,11 +11,13 @@ class AutoConvHandler:
     def __init__(self, conversation, telegram_state_name):
         self.conversation = conversation
         self.NEXT = telegram_state_name
-        self.tData = TelegramData()
+        self.tData = TelegramData(conversation.users_list)
         self.prev_state = None
         self.curr_state = conversation.start
-        self._bkup_state_routes, self._list_keyboard = None, None
+        self._bkup_routes, self._list_keyboard = None, None
         self.list_labels = None
+        self.conversation._check_routes()
+        self.conversation._set_states_text()
 
     # ---- Next state
 
@@ -83,14 +85,14 @@ class AutoConvHandler:
             data_context.update({"error": False})
         elif data_context.get("error") is not None:
             data_context.pop("error")
-        self._restore_basic_routes(self.prev_state)
+        self._restore_basic_routes()
         return new_state
 
     def _wrong_message(self, text=True):
         """Handler for wrong message"""
         self.tData.update.message.delete()
         state = self.conversation.get_state(self.tData.context.user_data.get("state"))
-        self._restore_basic_routes(state)
+        self._restore_basic_routes()
         error_text = text and state.regex_error_text or state.handler_error_text
         if error_text and not self.tData.context.user_data.get("error"):
             self.tData.context.user_data.update({"error": True})
@@ -125,7 +127,7 @@ class AutoConvHandler:
     def _build_dynamic_list(self, state, keyboard):
         """Build dynamic list for current state"""
         if self.prev_state != self.curr_state:
-            self._bkup_state_routes = self.conversation.routes.get(state.name)
+            self._bkup_routes = (state.name, self.conversation.routes.get(state.name))
         data = self.tData.context.user_data
         state_l = data.get("list")
         basic_routes = {
@@ -138,26 +140,18 @@ class AutoConvHandler:
                 if (cd := button.callback_data) is not None and isinstance(cd, int):
                     button.callback_data += len(state_l)
         self.list_labels = state.list_labels and state.list_labels(self.tData.prepare())
+        maxr, btns = state.list_max_row, state.list_buttons
         list_buttons = [
             [
                 InlineKeyboardButton(
-                    self.list_labels[r * state.list_max_row + i]
-                    if self.list_labels
-                    else b,
-                    callback_data=r * state.list_max_row + i,
+                    self.list_labels[r * maxr + i] if self.list_labels else button,
+                    callback_data=r * maxr + i,
                 )
-                for i, b in enumerate(
-                    state_l[
-                        r * state.list_max_row : r * state.list_max_row
-                        + state.list_max_row
-                    ]
-                    if state.list_all
-                    else state.list_buttons
+                for i, button in enumerate(
+                    state_l[r * maxr : r * maxr + maxr] if state.list_all else btns
                 )
             ]
-            for r in range(
-                ceil((len(state_l) if state.list_all else 1) / state.list_max_row)
-            )
+            for r in range(ceil((len(state_l) if state.list_all else 1) / maxr))
         ]
         keyboard = list_buttons + keyboard
         self._list_keyboard = keyboard
@@ -189,13 +183,19 @@ class AutoConvHandler:
         keyboard, size = keyboard if isinstance(keyboard, tuple) else (keyboard, None)
         state.add_keyboard(keyboard, size, max_row=state.max_row)
 
+    def _refresh_auth_users(self, state):
+        """Refresh authorized users"""
+        new_users = state.refresh_auth(self.tData.prepare())
+        self.conversation.users_list = new_users
+        self.tData.users = new_users
+
     def _build_dynamic_stuff(self, state):
-        """Compute dynamic stuff: action > routes > keyboard > list"""
+        """Compute dynamic stuff: action > routes > keyboard > list > refresh"""
         data = self.tData.context.user_data
         if self.prev_state != self.curr_state and data.get("list"):
             data.pop("list")
             data.pop("list_i")
-            self.list_labels, self._bkup_state_routes = None, None
+            self.list_labels, self._bkup_routes = None, None
         if state.list:
             self._update_dynamic_list(state)
         if state.action:
@@ -207,25 +207,22 @@ class AutoConvHandler:
         keyboard = self._build_keyboard(state)
         if state.list:
             keyboard = self._build_dynamic_list(state, keyboard)
-        reply_msg = (
-            state.msg.replace("@@@", action_str)
-            if state.action and action_str
-            else state.msg
-        )
+        if state.refresh_auth:
+            self._refresh_auth_users(state)
+        reply_msg = state.msg.replace("@@@", state.action and action_str or "")
         return InlineKeyboardMarkup(keyboard), reply_msg
 
     # ---- Dev functions
 
-    def _restore_basic_routes(self, state):
-        if not self._bkup_state_routes:
+    def _restore_basic_routes(self):
+        if not self._bkup_routes:
             return
-        back_route = self._bkup_state_routes.pop("BACK", None)
-        default_route = self._bkup_state_routes.pop(-1, None)
+        state, routes = self._bkup_routes
         self.conversation.add_routes(
-            state,
-            self._bkup_state_routes,
-            default=default_route,
-            back=back_route,
+            self.conversation.get_state(state),
+            routes,
+            default=routes.pop(-1, None),
+            back=routes.pop("BACK", None),
         )
 
     def _init_context(self, state):
@@ -261,6 +258,12 @@ class AutoConvHandler:
             self.tData.context.user_data.update({"bot-msg": new_msg})
         return new_msg
 
+    def _do_operations(self, state, data):
+        if state.operations and data in state.operations:
+            state.operations.get(data)(self.tData.prepare())
+            return True
+        return False
+
     # ---- Public functions
 
     def force_state(self, state, update):
@@ -329,7 +332,7 @@ class AutoConvHandler:
                 data = (
                     state.handler
                     and state.handler(self.tData.prepare())
-                    or self.tData.update.message.text
+                    or (not state.handler and self.tData.update.message.text)
                 )
                 if state.regex and not match(state.regex, str(data)) or not data:
                     return self._wrong_message(state.regex)
@@ -337,7 +340,8 @@ class AutoConvHandler:
                 self.tData.update.message.delete()
             # ---- next stage
             typed_data = state.data_type(data) if data != "BACK" else "BACK"
-            state = self._change_state(typed_data)
+            if not self._do_operations(state, typed_data):
+                state = self._change_state(typed_data)
             self._send_message(state, to_reply, save=False)
             if state == self.conversation.end:
                 self.tData.context.user_data.clear()
